/************************************************************
 *
 * Connection Controller
 *
 ************************************************************/

Ext.define('POSync.controller.Connection', {
    extend: 'Ext.app.Controller',
    id: 'Connection',

    refs: [
        { ref: 'panel', selector: '#panel'},
        { ref: 'projects', selector: '#panel #projects'},
        { ref: 'files', selector: '#panel #files'},
        { ref: 'operations', selector: '#panel #operations'},
        { ref: 'localDir', selector: '#panel #localdir'},
    ],

    /* This is called before the viewport is created */
    init: function() {
        console.log ('Connection.init');
        this.control({
        });
    },

    /* This is called after the viewport is created */
    onLaunch: function() {
        var me = this;
        console.log ('Connection.onLaunch: controller onLaunch');

        this.projectsStore = this.getProjects().getStore();
        this.filesStore = this.getFiles().getStore();
        this.localDirStore = this.getLocalDir().getStore();
        this.operationsStore = Ext.StoreManager.get('Operations');

        var app = this.getApplication();
        this.guiController = app.getController('Gui');

        // receives an even when clicking "sync" on Tray icon
        ipcRenderer.on('sync', (event, arg) => {
                console.log('sync: '+arg);
                me.sync();
        });
    },

    /* FIXME: no easy way to know if login is successful */
    login: function (request_options) {
		var me = this;
        console.log('Connection.login');

        /* Send the login request */
        var configData = this.guiController.configData;
        console.log ('Connection.login: url('+configData.url+') email('+configData.email+') password('+configData.password+')');
        Ext.Ajax.request({
            url: configData.url + '/intranet/auto-login-token',
            params: {
                email: configData.email,
                password: configData.password
            },
            callback: function (options, success, response) {
                if (!success) {
                        console.log('Connection.login: not successfull');
                        // FIXME: Write some message somewhere
                        return;
                }
                var responseJsonString = response.responseText;
                var responseObject = JSON.parse(responseJsonString);
                if (!responseObject) {
                        console.log('Connection.login: error parsing responseText='+responseText);
                        // FIXME: Write some message somewhere
                        return;
                }
                var token = responseObject.token;
                if (!token) {
                        console.log('Connection.login: no token in responseText='+responseText);
                        // FIXME: Write some message somewhere
                        return;
                }
                console.log('Connection.login: token='+token);
                configData.token = token;

                // load the list of projects available to the user
                me.loadProjects();
            }
        });
    },

    /* Send a request to get the list of projects */
    loadProjects: function () {
        var me = this;
        console.log('Connection.loadProjects:');

        // Check if already there
        if (this.projectsStore.isLoaded) return;

        var configData = this.guiController.configData;
        var proxy = this.projectsStore.getProxy();
        var baseUrl = configData.url;

        proxy.url = baseUrl+"/intranet-rest/im_project";
        proxy.extraParams = {
            format:             'json',
            query:              'parent_id is NULL'     // Select only top-level projects
        };

        this.projectsStore.load({
            callback: function (records,operation,success) {
                if (!success) {
                        console.log('Connection.loadProjects: not successfull');
                        // FIXME: Write some message somewhere
                        return;
                }
                me.projectsStore.isLoaded = success;            // mark the store as loaded
            }
        });
    },

    /* Send requests to load the files of all projects */
    loadFiles: function (projectId, successCallback) {
        var me = this;
        console.log('Gui.loadFiles: project_id='+projectId);

        // Skip if already loaded
        //if (''+me.filesStore.isLoaded == ''+projectId) return;

        var configData = this.guiController.configData;
        var proxy = me.filesStore.getProxy();
        var baseUrl = configData.url;
        proxy.url = baseUrl+"/intranet-rest-fs-openacs/file-tree.json";
        proxy.extraParams = {
            format:             'json',
            object_id:          projectId
        };

        me.filesStore.load({
            callback: function (records,operation,success) {
            	if (!success) {
                	console.log('Connection.login: not successfull');
                    // FIXME: Write some message somewhere
                    return;
                }
                me.filesStore.isLoaded = projectId;

                // Execute callback
                successCallback && successCallback.call(me, projectId);
            }
        });

    },

    /*
     * Get the local directory data and load the LocalFiles store.
     * directory-tree provides a tree with the following fields:
     */
    loadLocalDirectory: function (projectId) {
        console.log('Connection.loadLocalDirectory: '+projectId);

        // Skip if already loaded
        // if (''+this.localDirStore.isLoaded == ''+projectId) return;

        var configData = this.guiController.configData;
        if (!this.projectsStore.isLoaded) { alert('Projects not yet loaded'); return; }

        // Get the project path
        var project = this.projectsStore.getById(''+projectId);
        var path = configData.topdir+'/'+project.get('project_path');

        // make sure the path exists
        var fs = require('fs');
        if (!fs.existsSync(path)) { fs.mkdir(path); }

        // Get the contents
        const dirTree = require ('directory-tree');
        const tree = dirTree (path);
        this.fixLocalTree (tree);
        this.localDirStore.loadTree (tree);

        this.localDirStore.isLoaded = projectId;
    },

    /* Fix a tree before loading it */
    fixLocalTree: function (tree) {
        if (tree) {
            if (tree.type == 'directory') {
                tree.type = 'folder';
                tree.size = '';
                tree.expandable = true;
                tree.expanded = tree.children && tree.children.length > 0;
                tree.leaf = false;
                tree.mime_type = '';
            } else {
                const cp = require('child_process');
                // tree.type = cp.execFileSync('/usr/bin/file', ['--brief', '--mime-type', tree.path]).toString();
                // tree.sha1 = cp.execFileSync('/usr/bin/sha1sum', [tree.path]).toString().substr(0,40);
                // tree.date = cp.execFileSync('/usr/bin/stat', ['-c', '%y', tree.path]).toString();

                tree.type = 'unknown';
                tree.sha1 = '<sha1>';
                tree.date = '<date>';

                tree.expandable = false;
                tree.expanded = false;
                tree.leaf = true;
                tree.mime_type = tree.type;
            }
            tree.iconCls = 'icon-' + tree.type;
            if (tree.children) {
                var me = this;
                tree.children.forEach (function(child) {
                    me.fixLocalTree (child);
                });
            }
        }
    },

    /**
     * Start the synchronization
     * by loading the files store from the server.
     */
    sync: function (projectId) {
		var me = this;
        console.log('Connection.sync');

	    // Load the list of files and continue with syncCallback
        this.loadFiles(projectId, me.syncCallback);
	},

    /**
     * The list of files has arrived.
     * Now we can start with the actual sync
     */
	syncCallback: function(projectId) {
		var me = this;
        console.log('Connection.syncCallback');

		// Clean up the operations store
		me.operationsStore.removeAll();

        var root = me.filesStore.getRootNode();

	    if (''+me.projectsStore.isLoaded != ''+projectId) this.loadLocalDirectory(projectId);
	    root.cascadeBy(function(file) {

                // Exclude folders and the root
                if ("folder" == file.get('type')) return;
                if ("root" == file.get('id')) return;

                var pathArray = file.getPathArray();
                console.log('Connection.sync: path='+pathArray);
                console.log(file);

                var localFileFound = me.localDirStore.searchUsingPathArray(pathArray);
                console.log(localFileFound);

                if (!localFileFound) {
                        var operation = Ext.create('POSync.model.Operation', {
                                id: 'op_'+file.get('id'),
                                file_id: file.get('id'),
                                file_path: pathArray.join('/'),
                                file_name: file.get('name'),
                                mime_type: file.get('mime_type'),
                                file_sha1: null,
                                op: 'create'
                        });
                        me.operationsStore.add(operation);
                }
        });

        me.syncExecOperations(projectId);
    },



    /**
     * Takes a store with sync "Operations" and executes them.
     * This performs the actual sync.
     */
	syncExecOperations: function(projectId) {
		var me = this;
        console.log('Connection.syncExecOperations');

		me.operationsStore.each(function(op) {
  			console.log('Connection.syncExecOperations: op');
  			console.log(op);

  			var operation = op.get('op');
  			switch (operation) {
			case 'create':
				me.syncCreate(projectId, op);
				break
			default:
				alert('syncExecOperations: Internal error: unknown operation='+operation);
			}
		});

    },

	/**
	 * Download a single file.
	 *
	 * Called from syncExecOperations, which in turn
	 * loops through the list of operations created during
	 * sync.
	 */
    syncCreate: function(projectId, op) {
		var me = this;
        console.log('Connection.syncCreate');
        console.log(op);
        var configData = this.guiController.configData;
        var fileId = op.get('file_id');
        var filePath = op.get('file_path');
        var fileName = op.get('file_name');
        var mimeType = op.get('mime_type');

        // Get the project path
        var project = this.projectsStore.getById(''+projectId);
        var projectPath = configData.topdir+'/'+project.get('project_path');
		var path = projectPath+'/'+filePath;

        // make sure the path exists
        var fs = require('fs');
        if (!fs.existsSync(path)) { fs.mkdir(path); }

		var requestUrl = configData.url + '/intranet-rest-fs-openacs/download?file_id='+fileId;
		var xhr = new XMLHttpRequest();
		xhr.open("GET", requestUrl);
		xhr.responseType = "arraybuffer";
		// xhr.responseType = "blob";
		xhr.onload = function () {
			// Asynchronously saving files after they have been downloaded
    		if (this.status === 200) {
            	console.log('Connection.syncCreate: successful download - saving');

				// Decode the response encoded by server in base64
				var responseBuffer = xhr.response;
				var buf64 = new Buffer(responseBuffer, 'base64'); // decode

    		    // make sure the path exists and write the file
				var fs = require('fs');
		        if (!fs.existsSync(path)) { fs.mkdir(path); }
				fs.writeFile(path+"/"+fileName, buf64, function() {});

            	console.log('Connection.syncCreate: written file='+filePath);
    		} else {
            	console.log('Connection.syncCreate: download status='+this.status);
			}
		};
		xhr.send();
	}

/* End of POSync.controller.Connection */
});
